import gzip
from Bio.SeqIO.QualityIO import FastqGeneralIterator

def fill_template(template: str, read: str, count: str) -> str:
    '''
    Replace the placeholders in the template with the provided values

    Args:
        template (str): Template string
        read (int): Read number
        count (int): Number of times the read was observed

    Returns:
        str: Formatted string
    '''
    if "READ" not in template or "COUNT" not in template:
        raise ValueError("Template must contain READ and COUNT placeholders")
    elif template.count("READ") > 1 or template.count("COUNT") > 1:
        raise ValueError("Template must contain only one READ and COUNT placeholder")
    elif template.split('READ')[0].strip('>') == template.split('READ')[1].split('COUNT')[0]:
        raise ValueError(f"Template ({template}) read and count must have unique delimiters")

    return template.replace("READ", read).replace("COUNT", count)


def collapse(infile: str, outfile: str, format=f">seqREAD_xCOUNT") -> None:
    '''
    Collapse identical reads and return a fasta file with the collapsed reads

    Output header takes the format of ">seqREAD_xCOUNT" where READ is the read 
    number and COUNT is the number of times the read was observed

    To provide a custom header format, use the format argument including READ and COUNT
    as placeholders for the read number and count respectively. Everything before 

    Args:
        infile (str): Path to the input fasta file (can be gzipped)
        outfile (str): Path to the output fasta file
        format (str, optional): Custom header format. Defaults to ">seqREAD_xCOUNT".
    
    Returns:
        None
    '''
    if ">" not in format:
        raise ValueError("Header format must contain '>' character")
    
    unique_reads = {}

    # handle gzipped files
    with open(infile, 'rb') as f:
        magic_number = f.read(2)
        if magic_number == b'\x1f\x8b':
            f = gzip.open(infile, 'rt')
        else:
            f.seek(0)
            f = open(infile, 'r')

        # iterate over the input file recording the number of times each sequence is observed
        print(f"Reading input file {infile}...")
        for title, sequence, quality in FastqGeneralIterator(f):
            if sequence in unique_reads:
                unique_reads[sequence]['count'] += 1
            else:
                unique_reads[sequence] = {}
                unique_reads[sequence]['count'] = 1

    # write the collapsed reads to the output file using the provided header format
    if outfile.endswith('.gz'):
        f = gzip.open(outfile, 'wt')
    else:
        f = open(outfile, 'w')

    read_number = 1
    for seq, vals in unique_reads.items():
        f.write(f"{fill_template(format, str(read_number), str(vals['count']))}\n")
        f.write(f"{seq}\n")
        read_number += 1



from RDPTools.collapse import collapse

import gzip
import hashlib
import pytest


def test_collapse_fq():
    collapse(
        'tests/test_data/reads.fq',
        'tests/test_data/reads.collapsed.fa',
    )
    with open('tests/test_data/reads.collapsed.fa', 'rb') as f:
        m = hashlib.md5()
        m.update(f.read())
        assert m.hexdigest() == '1d2cc643a8ca7642e68964fae580b56b'

def test_collapse_fq_gz():
    collapse(
        'tests/test_data/reads.fq.gz',
        'tests/test_data/reads.collapsed.fa.gz',
    )
    # these files must be read differently as gzip files from python automatically
    # add the metadata (no -n flag) so MD5 hashes will not match
    with gzip.open('tests/test_data/reads.collapsed.fa.gz', 'rb') as f:
        content = f.read()
    
    md5_hash = hashlib.md5(content).hexdigest()
    assert md5_hash == '1d2cc643a8ca7642e68964fae580b56b'

def test_collapse_format():
    collapse(
        'tests/test_data/reads.fq',
        'tests/test_data/reads.collapsed.fa',
        format='>testREAD_xCOUNT',
    )
    with open('tests/test_data/reads.collapsed.fa', 'rb') as f:
        m = hashlib.md5()
        m.update(f.read())
        assert m.hexdigest() == 'bb860f18c24e3a1278acd6a09f6103b8'

def test_collapse_error_raised_unqi_delims():
    with pytest.raises(ValueError, match="read and count must have unique delimiters"):
        collapse(
            'tests/test_data/reads.fq',
            'tests/test_data/reads.collapsed.fa',
            format='>testREADtestCOUNT',
        )
        with open('tests/test_data/reads.collapsed.fa', 'rb') as f:
            m = hashlib.md5()
            m.update(f.read())

def test_collapse_error_raised_missing_delim():
    with pytest.raises(ValueError, match="Template must contain READ and COUNT placeholders"):
        collapse(
            'tests/test_data/reads.fq',
            'tests/test_data/reads.collapsed.fa',
            format='>testtestCOUNT',
        )
        with open('tests/test_data/reads.collapsed.fa', 'rb') as f:
            m = hashlib.md5()
            m.update(f.read())

def test_collapse_error_raised_double_read():
    with pytest.raises(ValueError, match="Template must contain only one READ and COUNT placeholder"):
        collapse(
            'tests/test_data/reads.fq',
            'tests/test_data/reads.collapsed.fa',
            format='>testREADtestCOUNT_READ',
        )
        with open('tests/test_data/reads.collapsed.fa', 'rb') as f:
            m = hashlib.md5()
            m.update(f.read())


#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages
from pathlib import Path

with open("README.rst") as readme_file:
    readme = readme_file.read()

# Get the path to the requirements.txt file
requirements_path = Path(__file__).parent / 'requirements.txt'

# Read the contents of the requirements file
with open(requirements_path) as f:
    requirements = f.read().splitlines()

test_requirements = [
    "pytest>=3",
]

setup(
    author="Jack Tierney",
    author_email="jackcurragh@gmail.com",
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    description="A python command-line utility for working with RiboSeq.Orgs Data Portal",
    entry_points={
        "console_scripts": [
            "RDP-Tools=RDPTools.RDPTools:rdp_tools",
        ],
    },
    install_requires=requirements,
    license="MIT license",
    include_package_data=True,
    keywords="RDP-Tools",
    name="RiboSeq-DP-Tools",
    packages=find_packages(
        include=["RDPTools", "RDPTools.*"],
        exclude=[
            "sample_data/*",
        ],
    ),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/riboseqorg/RDP-Tools",
    version="0.1.3",
    zip_safe=False,
)
